-- rs232 transmit code


LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE IEEE.std_logic_unsigned.ALL;


ENTITY rs232_tx IS

PORT
(
	clk						: IN	STD_LOGIC;
	baudClkEn				: IN 	STD_LOGIC;  -- RS232 Baud Rate clk enable.
	reset					: IN	STD_LOGIC;
-- TX SIGNALS
	txRequest				: IN	STD_LOGIC ;  -- internal request to write data to ftdi chip
	cts						: IN	STD_LOGIC  ; -- clear to send from USB device
	dataOut					: IN STD_LOGIC_VECTOR( 7 DOWNTO 0);
	wrComplete				: OUT	STD_LOGIC;	-- internal signal that write to ftdi chip is complete
	txd						: OUT	STD_LOGIC;	-- transmit bit stream to USB device
	test					: OUT STD_LOGIC_VECTOR ( 7 DOWNTO 0)

);
							
end rs232_tx;

ARCHITECTURE rtl OF rs232_tx IS


	SIGNAL RS232OUTLatch	: STD_LOGIC_VECTOR( 9 DOWNTO 0);
	SIGNAL txBitCounter		: STD_LOGIC_VECTOR( 3 DOWNTO 0);
	SIGNAL txBitCounterEn	: STD_LOGIC;
	SIGNAL txBitCountDone	: STD_LOGIC;
	SIGNAL txLoad 			: STD_LOGIC;
	SIGNAL txShift 			: STD_LOGIC;
	SIGNAL TXingNow 		: STD_LOGIC;
	SIGNAL iWrComplete		: STD_LOGIC;	-- internal signal that write to ftdi chip is complete



	TYPE txStates IS 	(STXIDLE,STXLOAD,STXWAITCTS,STXWAITBAUDCLK,
	STXSTARTBIT,STXSENDING,STXSHIFT,STXDONE,STXDONEWAIT1,STXDONEWAIT2,
	STXDONEWAIT3,STXDONEWAIT4,STXDONEWAIT5);
	SIGNAL txCurrentState 	: txStates;
	SIGNAL txNextState 		: txStates;


BEGIN

----------------------------RS232 data transmitter ---------------------------------


wrComplete <= iWrComplete;

-- use txRequest active to latch internal data bus for serialiasation for RS232
-- need 8 bit shift register for RS232 output 
-- needs parallel load

-- here it is 
l_tx_shifter : PROCESS( clk, reset,txLoad, txShift)
	BEGIN
		IF ( reset = '1' ) THEN 
			RS232OUTLatch <= '1' & x"FF" & '1';
		ELSIF ( rising_edge(clk)) THEN
			IF ( txLoad = '1' ) THEN
				RS232OUTLatch <= '1' & dataOut & '0' ; 		-- load data to send plus start and stop bits.
			ELSIF ( txShift = '1') THEN
				RS232OUTLatch( 9 DOWNTO 0 ) <=  '1' & RS232OUTLatch( 9  DOWNTO 1 );
			ELSE
				RS232OUTLatch <= RS232OUTLatch;
			END IF;
		END IF ;
	END PROCESS;
	
-- now just need to look for txRequest and wait for CTS to be active. Then clock out 9 bits of data.

l_tx_bit_counter: 
PROCESS ( clk,reset,txBitCounterEn, baudClkEn )
	BEGIN
		IF ( reset = '1' ) THEN 
			txBitCounter <= x"0";
			txBitCountDone <= '0';
		ELSIF ( rising_edge(clk)) THEN
			IF ( baudClkEn = '1' AND txBitCounterEn = '1' AND txBitCounter < "1001" ) THEN
				txBitCounter <= txBitCounter + 1 ;
			ELSIF ( txBitCounterEn = '0' ) THEN
				txBitCounter <= x"0";			
			END IF;
			IF ( txBitCounter = "1001" ) THEN
				txBitCountDone <= '1';
			ELSE
				txBitCountDone <= '0';
			END IF;
		END IF ;
	END PROCESS;

-- tx state machine

l_tx_states:	
PROCESS ( reset, clk, txCurrentState, txRequest,cts)
	BEGIN
	IF ( reset ='1' ) THEN 
		txNextState <= STXIDLE;
	ELSIF  ( rising_edge(clk)) THEN
		CASE txNextState IS
			WHEN STXIDLE =>
-- look for TX request and CTS being active - then start to send
				TXLoad <='0';
				TXSHIFT <='0';	
				iWrComplete <= '0';
				TXingNow <='0';	
				txBitCounterEn <='0';
				IF (  txRequest = '1' ) THEN
					txNextState <= STXLOAD;
				END IF;
-- first grab data to transmit - also loads up START Aand STOP bits				
			WHEN STXLOAD =>
				TXLoad <='1';
				TXSHIFT <='0';
				TXingNow <='0';	
				iWrComplete <= '0';
				txBitCounterEn <='0';
				txNextState <= STXWAITCTS;
-- wait for CTS active
			WHEN STXWAITCTS =>
				TXLoad <='0';
				TXSHIFT <='0';
				TXingNow <='0';	
				iWrComplete <= '0';
				txBitCounterEn <='0';
				IF ( cts = '1'  ) THEN
					txNextState <= STXWAITBAUDCLK;
				END IF;
-- wait for next baud clk enable to start sending proper				
			WHEN STXWAITBAUDCLK =>
				TXLoad <='0';
				TXSHIFT <='0';
				TXingNow <='0';	
				txBitCounterEn <='0';
				iWrComplete <= '0';
				IF ( baudClkEn = '1') THEN
					txNextState <=STXSTARTBIT;
				END IF;
-- got baudrateClk so send out START bit
			WHEN STXSTARTBIT =>
				TXLoad <='0';
				TXSHIFT <='0';
				TXingNow <='1';	
				iWrComplete <= '0';
				txBitCounterEn <='1';		-- enable bit counter
				txNextState <=STXSENDING;
-- sending data - so wait for next baud clock
			WHEN STXSENDING =>
				TXLoad <='0';
				TXSHIFT <='0';
				TXingNow <='1';	
				txBitCounterEn <='1';		-- enable bit counter
				iWrComplete <= '0';
				IF ( baudClkEn = '1') THEN
					txNextState <=STXSHIFT;
				END IF;
-- sending data -have we done - no so shift - yes finish off nicley.
			WHEN STXSHIFT =>
				TXLoad <='0';
				TXSHIFT <='1';				-- shift by 1 bit
				TXingNow <='1';	
				txBitCounterEn <='1';		-- enable bit counter
				iWrComplete <= '0';
				IF ( txBitCountDone  = '1') THEN
					txNextState <=STXDONE;
				ELSE
					txNextState <=STXSENDING;
				
				END IF;


-- finished sending data - now wait for 2 bits times
			WHEN STXDONE =>
				TXLoad <='0';
				TXSHIFT <='0';
				TXingNow <='0';	
				txBitCounterEn <='0';
				iWrComplete <= '0';   
				IF ( baudClkEn = '1') THEN
					txNextState <=STXDONEWAIT1;
				END IF;

			WHEN STXDONEWAIT1 =>
				TXLoad <='0';
				TXSHIFT <='0';
				TXingNow <='0';	
				txBitCounterEn <='0';
				iWrComplete <= '0';   
				IF ( baudClkEn = '1') THEN
					txNextState <=STXDONEWAIT2;
				END IF;
			WHEN STXDONEWAIT2 =>
				TXLoad <='0';
				TXSHIFT <='0';
				TXingNow <='0';	
				txBitCounterEn <='0';
				iWrComplete <= '0';   
				IF ( baudClkEn = '1') THEN
					txNextState <=STXDONEWAIT3;
				END IF;
			WHEN STXDONEWAIT3 =>
				TXLoad <='0';
				TXSHIFT <='0';
				TXingNow <='0';	
				txBitCounterEn <='0';
				iWrComplete <= '0';
				IF ( baudClkEn = '1') THEN
					txNextState <=STXDONEWAIT4;
				END IF;
			WHEN STXDONEWAIT4 =>
				TXLoad <='0';
				TXSHIFT <='0';
				TXingNow <='0';	
				txBitCounterEn <='0';
				iWrComplete <= '1';   -- signal that write is complete
				txNextState <=STXDONEWAIT5;
-- wait for tx_request to go away
			WHEN STXDONEWAIT5 =>
				TXLoad <='0';
				TXSHIFT <='0';
				TXingNow <='0';	
				txBitCounterEn <='0';
				iWrComplete <= '0';   -- signal that write is complete
				IF (  txRequest = '0' ) THEN
					txNextState <=STXIDLE;
				END IF;
			WHEN OTHERS =>
				TXLoad <='0';
				TXSHIFT <='0';
				TXingNow <='0';	
				iWrComplete <= '0';
				txBitCounterEn <='0';
				txNextState <=STXIDLE;
		END CASE;
	END IF;
END PROCESS;			
				
-- output data				
txd <= '1' WHEN TXingNow ='0'
	ELSE 	RS232OUTLatch(0);
			
-- tets stuff			
			
			
test(0) <= txRequest;

test(1) <= iWrComplete;


END rtl;







