-- USB bus arbitor
-- routine gives fair access to the external USB bus as required.


LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE IEEE.std_logic_unsigned.ALL;


ENTITY busArb IS

PORT
(
	clk						: IN	STD_LOGIC;
	busclk					: IN	STD_LOGIC;
	reset					: IN	STD_LOGIC;
	wrComplete				: OUT	STD_LOGIC;	-- internal signal that write to ftdi chip is complete

	txNotFull				: IN	STD_LOGIC;  -- fifo status  signal from ftdi chip
	rxNotEmpty				: IN	STD_LOGIC;  -- fifo status  signal from ftdi chip
	rd						: OUT	STD_LOGIC;  -- fifo control  signal to ftdi chip
	rdEn					: OUT	STD_LOGIC;  -- internal siganl indicates that ftdi read data is valid on bus
	wr						: OUT	STD_LOGIC;  -- fifo control  signal to ftdi chip
	txRequest				: IN	STD_LOGIC   -- internal request to write data to ftdi chip
);

END busArb;

ARCHITECTURE rtl OF busArb IS

	
	CONSTANT endWaitCount 	: STD_LOGIC_VECTOR( 3 DOWNTO 0 ) := "0110" ;
	CONSTANT endAccessCount 	: STD_LOGIC_VECTOR( 3 DOWNTO 0 ) := "1010" ;
	CONSTANT endCycleCount 	: STD_LOGIC_VECTOR( 3 DOWNTO 0 ) := "0110" ;

	SIGNAL cycleAccessCount  	: STD_LOGIC_VECTOR( 3 DOWNTO 0 );
	SIGNAL cycleCount  		: STD_LOGIC_VECTOR( 3 DOWNTO 0 );
	SIGNAL cycleWaitCount  	: STD_LOGIC_VECTOR( 3 DOWNTO 0 );
	SIGNAL enableAccessCount 	: STD_LOGIC;
	SIGNAL enableWaitCount 	: STD_LOGIC;
	SIGNAL enableCycleCount 	: STD_LOGIC;
	
	
 BEGIN
 l_statemachine : BLOCK

--   STD_LOGIC_VECTOR( 2 DOWNTO 0 );
--  CONSTANT TXREQUIRED : states := "000";
--	CONSTANT RXREQUIRED : states := "001";
--	CONSTANT DOTX 		: states := "010";
--	CONSTANT DOTX1 		: states := "011";
--	CONSTANT DORX 		: states := "100";
--	CONSTANT DORX1 		: states := "101";
--	CONSTANT RXWAIT 	: states := "110";
--	CONSTANT TXWAIT 	: states := "111";

	TYPE states IS 	( TXREQUIRED, RXREQUIRED, DOTX, DOTX1, DORX, DORX1, RXWAIT, RXWAIT0, TXWAIT,TXDONE,TXDONE0);

	SIGNAL currentState : states;
	SIGNAL nextState :	 states;
-- state machine
	BEGIN
l_trans:		PROCESS ( reset,clk, currentState)
		BEGIN
			IF ( reset ='1' ) THEN 
				currentState <= RXREQUIRED;
			ELSIF  ( rising_edge(clk)) THEN
					currentState <= nextState;
			END IF;
		END PROCESS;

	-- state transitions
l_states:	PROCESS ( currentState,rxNotEmpty,txNotFull,txRequest,cycleAccessCount,cycleWaitCount,cycleCount)
		BEGIN
			CASE currentState IS
				WHEN RXREQUIRED =>
					IF ( rxNotEmpty ='1' ) THEN
						nextState <= DORX;
					ELSE
						nextState <= TXREQUIRED ;
					END IF;
					rd <= '0';
					rdEn <='0';
					wr <='0';
					enableAccessCount <='0';
					enableWaitCount <='0';
					enableCycleCount <='0';
					wrComplete <='0';

				WHEN TXREQUIRED =>
					IF ( txNotFull = '1' AND txRequest = '1' ) THEN
						nextState <= DOTX;
					ELSE
						nextState <= RXREQUIRED ;
					END IF;
					rd <= '0';
					rdEn <='0';
					wr <='0';
					enableAccessCount <='0';
					enableWaitCount <='0';
					enableCycleCount <='0';
					wrComplete <='0';

				WHEN DORX =>
					IF ( cycleAccessCount >= endAccessCount ) THEN
						nextState <= DORX1 ;
					ELSE
						nextState <= DORX ;
					END IF;
					rd <= '1';
					rdEn <='0';
					enableAccessCount <='1';
					enableCycleCount <='0';
					enableWaitCount <='0';
					wr <='0';
					wrComplete <='0';
					-- data valid on internal buses
				WHEN DORX1 =>
					nextState <= RXWAIT ;
					rd <= '1';
					rdEn <= '1';
					wr <='0';
					wrComplete <='0';
					enableAccessCount <='0';
					enableCycleCount <='0';
					enableWaitCount <='0';

	-- wait for internal RD processes to complete 
				WHEN RXWAIT =>
					IF (  cycleWaitCount >= endWaitCount ) THEN
						nextState <= RXWAIT0 ;
					ELSE
						nextState <= RXWAIT ;
					END IF;
					rdEn <='0';
					rd <= '1';
					wr <='0';
					wrComplete <='0';
					enableWaitCount <='1';
					enableCycleCount <='0';
					enableAccessCount <='0';
	-- wait to allow for inter cycle gap 
				WHEN RXWAIT0 =>
					IF (  cycleCount >= endCycleCount ) THEN
						nextState <= TXREQUIRED ;
					ELSE
						nextState <= RXWAIT0 ;
					END IF;
					rdEn <='0';
					rd <= '0';
					wr <='0';
					wrComplete <='0';
					enableWaitCount <='0';
					enableCycleCount <='1';
					enableAccessCount <='0';
						
				WHEN DOTX =>
					IF ( cycleAccessCount >= endAccessCount ) THEN
						nextState <= DOTX1 ;
					ELSE
						nextState <= DOTX ;
					END IF;
					rdEn <='0';
					rd <= '0';
					wr <='1';
					wrComplete <='0';
					enableAccessCount <='1';
					enableCycleCount <='0';
					enableWaitCount <='0';

				WHEN DOTX1 =>
					nextState <= TXWAIT ;
					rdEn <='0';
					rd <= '0';
					wr <='1';
					wrComplete <='0';
					enableAccessCount <='0';
					enableCycleCount <='0';
					enableWaitCount <='0';
					

				WHEN TXWAIT =>
					IF (  cycleWaitCount >= endWaitCount ) THEN
						nextState <=  TXDONE;
					ELSE
						nextState <= TXWAIT ;
					END IF;
					rdEn <='0';
					rd <= '0';
					wr <='0';
					enableAccessCount <='0';
					enableCycleCount <='0';
					enableWaitCount <='1';
					wrComplete <='0';

				WHEN TXDONE =>
					nextState <= TXDONE0 ;
					wrComplete <='1';
					rdEn <='0';
					rd <= '0';
					wr <='0';
					enableAccessCount <='0';
					enableCycleCount <='0';
					enableWaitCount <='1';

					-- wait to allow for inter cycle gap 
				WHEN TXDONE0 =>
					IF (  cycleCount >= endCycleCount ) THEN
						nextState <= RXREQUIRED;
					ELSE
						nextState <= TXDONE0;
					END IF;
					rdEn <='0';
					rd <= '0';
					wr <='0';
					wrComplete <='0';
					enableWaitCount <='0';
					enableCycleCount <='1';
					enableAccessCount <='0';
					
				WHEN OTHERS =>
					nextState <= RXREQUIRED;
					rdEn <='0';
					rd <= '0';
					wr <='0';
					wrComplete <='0';
					enableCycleCount <='0';
					enableAccessCount <='0';
					enableWaitCount <='0';
			END CASE;
	END PROCESS;
END BLOCK;

l_cycleAccessCount :PROCESS ( enableAccessCount,clk)

BEGIN
	IF ( enableAccessCount ='0' ) THEN 
		cycleAccessCount <="0000";
	ELSIF   ( rising_edge(clk)) THEN
		cycleAccessCount <= cycleAccessCount + 1;
	END IF;
END PROCESS;

l_cycleWaitCount :PROCESS ( enableWaitCount,clk)

BEGIN
	IF ( enableWaitCount ='0' ) THEN 
		cycleWaitCount <="0000";
	ELSIF   ( rising_edge(clk)) THEN
		cycleWaitCount <= cycleWaitCount + 1;
	END IF;
END PROCESS;


l_cycleCount : PROCESS( enableCycleCount,clk)
BEGIN
	IF ( enableCycleCount ='0' ) THEN 
		cycleCount <="0000";
	ELSIF   ( rising_edge(clk)) THEN
		cycleCount <= cycleCount + 1;
	END IF;
END PROCESS;

END rtl;


