-- Decoder for packet exchange with PC
-- in coming packet format is
-- address,address,control1,control2,data(0)......data(n),csum
-- out going packet format is
-- byte_count,incoming_csum,data(0)......data(n),crc
--
-- the whole packet will be 64 bytes long
-- csum is an 8 bit addition of all data apart from headers
-- address, determines the destination of the data
-- a sync pattern of 16 sets of "AA" will precede any packet
-- when processed there will be an acknowledge packet sent.

-- control bits indicate the action of the incoming packet
-- Control1(0) = status request
-- 			Returns status 
-- Control1(1) = 	write data to control registers
--		

-- will respond with an ACK frame to all incoming Frames
-- ack format
--		header of 0x55 16 times over
--      byte count
--	    status - 16 bytes
--		csum
--

-- INCLUDE LIBRARY FILES 

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_arith.ALL;
USE IEEE.std_logic_unsigned.all;



ENTITY  packet_decoder IS
PORT(
	reset					: IN STD_LOGIC ;	
	ftdi_data				: IN STD_LOGIC_vector(7 downto 0) 	;	-- pc_data bus 	input
	status_data				: IN STD_LOGIC_vector(7 downto 0) 	;	-- data from status registers
	ftdi_data_rdy			: IN STD_LOGIC;
	sysClk					: IN STD_LOGIC ;
	tx_done					: IN STD_LOGIC ;	-- data sent to IO fifo

	control_wr_complete		: OUT STD_LOGIC;
	ftdi_wr					: OUT STD_LOGIC ;	-- write data to IO fifo
	ftdi_rd					: OUT STD_LOGIC ;	-- read data from IO fifo
	status_rd				: OUT STD_LOGIC ;
	control_wr				: OUT STD_LOGIC ;
	updating_now			: OUT STD_LOGIC ;
	data_out				: OUT STD_LOGIC_vector(7 downto 0) 	;	-- pc_data bus output 	
	packet_ram_data_out		: OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
	control_address			: OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
	test					: OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
	);
							
END ENTITY packet_decoder;

ARCHITECTURE rtl OF packet_decoder IS
-- CONSTANTS FOR OFFSETS INTO PACKET RAM
	CONSTANT ADDRESS_0		: STD_LOGIC_VECTOR (5 DOWNTO 0):= "000000";
	CONSTANT ADDRESS1 		: STD_LOGIC_VECTOR (5 DOWNTO 0):= "000000";
	CONSTANT ADDRESS2 		: STD_LOGIC_VECTOR (5 DOWNTO 0):= "000001";
	CONSTANT CONTROL1 		: STD_LOGIC_VECTOR (5 DOWNTO 0):= "000010";
	CONSTANT CONTROL2 		: STD_LOGIC_VECTOR (5 DOWNTO 0):= "000011";
	CONSTANT DATA_START		: STD_LOGIC_VECTOR (5 DOWNTO 0):= "000100";
	CONSTANT CSUM_ADDR		: STD_LOGIC_VECTOR (5 DOWNTO 0):= "111111";
	CONSTANT ACK_BYTE_COUNT	: STD_LOGIC_VECTOR (7 DOWNTO 0):= x"10";


	SIGNAL addr_reg 		: STD_LOGIC_vector(7 downto 0);
-- 	SIGNAL temp: STD_LOGIC_VECTOR(0 to 2 ) ;
	SIGNAL wr_this_pod 		: STD_LOGIC;
	SIGNAL rd_this_pod		: STD_LOGIC;
	SIGNAL packet_ram_addr	: STD_LOGIC_VECTOR (5 DOWNTO 0);
	SIGNAL i_packet_ram_data_out	: STD_LOGIC_VECTOR (7 DOWNTO 0);
	SIGNAL csum				: STD_LOGIC_VECTOR (7 DOWNTO 0);
	SIGNAL tx_csum			: STD_LOGIC_VECTOR (7 DOWNTO 0);
	SIGNAL send_count		: STD_LOGIC_VECTOR (7 DOWNTO 0);
	SIGNAL packet_header_found	: STD_LOGIC;
	SIGNAL sync_count		: STD_LOGIC_VECTOR (3 DOWNTO 0);
	SIGNAL reset_sync_count	: STD_LOGIC_VECTOR (4 DOWNTO 0);
	SIGNAL process_data		: STD_LOGIC;
	SIGNAL get_data_active	: STD_LOGIC;
	SIGNAL process_data_active 	: STD_LOGIC;
	SIGNAL packet_ram_wr	: STD_LOGIC;
	SIGNAL find_packet_start: STD_LOGIC;
	SIGNAL packet_found		: STD_LOGIC;
	SIGNAL data_count		: STD_LOGIC_VECTOR (5 DOWNTO 0);
	SIGNAL I_control_address: STD_LOGIC_VECTOR (7 DOWNTO 0);
	SIGNAL last_packet 		: STD_LOGIC_VECTOR (3 DOWNTO 0);
	TYPE STATE_TYPES IS ( 
		G_IDLE, 
		G_WAIT_FOR_BYTE_COUNT, 
		G_GET_DATA,G_GET_DATA_0,G_GET_DATA_1,G_GET_DATA_2,G_GET_DATA_3,
		G_GET_CSUM,G_GET_CSUM_0,
		G_CHECK_CSUM,
		G_SEND_CSUM,G_SEND_CSUM_0,G_SEND_CSUM_1,G_SEND_CSUM_2,
		G_USE_DATA,G_USE_DATA_0,G_USE_DATA_1,
		G_SEND_ACK,
		G_SEND_HEADER,G_SEND_HEADER_0,G_SEND_HEADER_1,G_SEND_HEADER_2,
		G_UPDATE_CONTROL,G_UPDATE_CONTROL_0,G_UPDATE_CONTROL_1,G_UPDATE_CONTROL_2,
		G_UPDATE_CONTROL_3,G_UPDATE_CONTROL_4,
		G_SEND_BYTE_COUNT,G_SEND_BYTE_COUNT_0,G_SEND_BYTE_COUNT_1,G_SEND_BYTE_COUNT_2,
		G_SEND_STATUS,G_SEND_STATUS_0,G_SEND_STATUS_1,G_SEND_STATUS_2,G_SEND_STATUS_3,
		G_SEND_STATUS_4,G_SEND_STATUS_5
		);

--	SIGNAL return_to			: STATE_TYPES;
	SIGNAL get_data_state			: STATE_TYPES;

	SIGNAL i_test			: STD_LOGIC_VECTOR (7 DOWNTO 0);


  component ram_dp
    generic (addr_bits		:integer;
             data_bits		:integer;
             register_out_flag	:integer := 0);
    port (reset		:in  std_logic;
        wr_clk	:in  std_logic;
    	wr_en	    	:in  std_logic;
        wr_addr	:in  std_logic_vector (addr_bits-1 downto 0);
        wr_data	:in  std_logic_vector(data_bits-1 downto 0);
		rd_clk	:in  std_logic;
        rd_addr	:in  std_logic_vector (addr_bits-1 downto 0);
        rd_data	:out std_logic_vector(data_bits-1 downto 0)
         );
  end component;

BEGIN

-- find the start of a packet by looking for 16 lots of "AA" in the data
l_find_packet_start:
PROCESS (reset,sysclk,find_packet_start,ftdi_data_rdy, ftdi_data  )
BEGIN
	IF ( reset = '1'  ) THEN
		packet_found <='0';
		sync_count <= x"0";
	ELSIF ( rising_edge(sysclk)) THEN
		-- count consecutive sync pulses
			IF (  find_packet_start ='1') THEN
				IF ( ftdi_data_rdy = '1' ) THEN
					IF ( ftdi_data = x"AA" ) THEN
						IF ( sync_count =  x"F") THEN
							packet_found <= '1';
						ELSE	
							packet_found <= '0';
							sync_count <= sync_count + 1 ;
						END IF;
					ELSE 
						sync_count <= x"0";
					END IF;
				END IF;
			ELSE
				sync_count <= x"0";
				packet_found <= '0';
			END IF;
		-- got 16 so that means start of packet
	END IF;
END PROCESS;	

-- test(0) <= '1' WHEN ftdi_data = x"AA" else '0';

-- write 64 bytes of data into packet ram and calculate the CSUM as we go
l_write_packet_data: 
PROCESS (reset,sysclk,packet_found,ftdi_data_rdy, ftdi_data  )
BEGIN
	IF ( reset = '1' ) THEN
		packet_ram_addr		<= "000000";	
--		packet_ram_data_in	<= x"00";	
		I_control_address		<= x"00";
		get_data_state <= G_IDLE;
		csum <= x"ff";
		control_wr_complete <= '0';
		process_data <='0';
		packet_ram_wr <='0';		
		find_packet_start <='0';
		get_data_active <='0';
--		send_count <= 
		status_rd <='0';
		control_wr<='0';
		find_packet_start <='1';
		ftdi_wr <= '0';
		last_packet <= "1111";
		i_test	<= x"00";
		updating_now <='0';
		ELSIF ( rising_edge(sysclk)) THEN
		CASE get_data_state IS

-- *************************** IDLE ************************************
			WHEN G_IDLE =>
				i_test <= x"00";
				packet_ram_wr <='0';
				packet_ram_addr	<="000000";
				process_data <='0';
				get_data_active <='0';
				status_rd <='0';
				control_wr_complete <= '0';
				find_packet_start <='1';
				control_wr<='0';
				find_packet_start <= '1';
				I_control_address		<= x"00";
				ftdi_wr <= '0';
				csum <= x"00";
				updating_now <='0';
				-- CHECK FINISHED WITH LAST PACKET 
				IF ( packet_found = '1' ) THEN
					get_data_state <= G_GET_DATA;
				END IF;
-- *************************** read data *******************************
			-- GET DATA BYTE AT A TIME
			WHEN G_GET_DATA =>
				i_test <= x"01";
				get_data_active <='1';
				find_packet_start <='0'; -- stop looking for packets until this one dealt with
				get_data_state <= G_GET_DATA_0;
			-- read data from ftdi
			WHEN G_GET_DATA_0 =>
				IF ( ftdi_data_rdy ='1') THEN
					get_data_state <= G_GET_DATA_1;
					packet_ram_wr <='1';
					IF ( 	packet_ram_addr = "11"& x"f" ) THEN
					ELSE
						csum <= csum + ftdi_data;
					END IF;
				END IF;
			-- data now on ftdi bus so wr to packet ram and update csum
			WHEN G_GET_DATA_1 =>
				i_test <= x"02";
				packet_ram_wr <='0';
				IF ( packet_ram_addr = "11"& x"f" ) THEN
					get_data_state <= G_GET_CSUM;
				ELSE
					packet_ram_addr <=packet_ram_addr + 1;
					get_data_state <= G_GET_DATA_0;
				END IF;
-- *************************** check checksum *******************************
			-- get address in RAM of CRC
			WHEN G_GET_CSUM =>
				i_test <= x"03";
				process_data <='1';

				packet_ram_addr <= CSUM_ADDR;
				get_data_state <= G_GET_CSUM_0;
			-- enable read of memory
			WHEN G_GET_CSUM_0 =>
				i_test <= x"04";
				get_data_state <= G_CHECK_CSUM;
			-- CHECK THAT CALC CSUM AND TRANSMITTED CSUM ARE THE SAME	
			WHEN G_CHECK_CSUM =>
				i_test <= x"05";
				IF ( i_packet_ram_data_out /= csum) THEN
			-- NOT SO DITCH PACKET 
					get_data_state <= G_IDLE;
				ELSE
					get_data_state <= G_USE_DATA;
				END IF;
			WHEN G_USE_DATA=>
				i_test <= x"06";
				process_data_active <= '1';
				packet_ram_addr <= CONTROL1;				
				get_data_state <= G_USE_DATA_0;
				-- wait for data to settle 
			WHEN G_USE_DATA_0 =>
				i_test <= x"07";
				get_data_state <= G_USE_DATA_1;
-- *************************** decide what to do with data**************
		-- go do what the control word says
			WHEN G_USE_DATA_1 =>
				i_test <= x"08";
				last_packet <= i_packet_ram_data_out(3 DOWNTO 0); -- DEBUG
				IF (i_packet_ram_data_out(0) ='1' ) THEN			-- STATUS REQUEST
					get_data_state <= G_SEND_ACK;
				ELSIF (i_packet_ram_data_out(1) ='1' ) THEN			-- CONTROL UPDATE
					get_data_state <= G_UPDATE_CONTROL;
				ELSE
					get_data_state <= G_SEND_ACK;					-- DON'T KNOW SO RETURN STATUS
					i_test(4) <= NOT i_test(4);
			END IF;				
-- *************************** updata control **************************                                             
			WHEN G_UPDATE_CONTROL  =>
				i_test <= x"09";
				updating_now <='1';
				i_test(3) <= NOT i_test(3);
				control_wr <='0';
				I_control_address		<= x"00";
				packet_ram_addr <= DATA_START;				
				get_data_state <= G_UPDATE_CONTROL_0;
			-- allow data to propergate
			WHEN G_UPDATE_CONTROL_0 =>
				i_test <= x"0a";
				get_data_state <= G_UPDATE_CONTROL_1;
			-- increment address
			WHEN G_UPDATE_CONTROL_1 =>
				i_test <= x"0b";
				control_wr <='1';
				get_data_state <= G_UPDATE_CONTROL_2;
			-- increment address
			WHEN G_UPDATE_CONTROL_2 =>
				i_test <= x"0c";
				control_wr <='0';
				I_control_address		<= I_control_address + 1 ;
				packet_ram_addr <= 	packet_ram_addr + 1;				
				get_data_state <= G_UPDATE_CONTROL_3;
			-- check if done
			WHEN G_UPDATE_CONTROL_3 =>
				i_test <= x"0d";
				IF ( packet_ram_addr = "11" & x"f") THEN
					get_data_state <= G_UPDATE_CONTROL_4;
				ELSE
					get_data_state <= G_UPDATE_CONTROL_0;
				END IF ;
			WHEN G_UPDATE_CONTROL_4 =>
				i_test <= x"0e";
				get_data_state <= G_SEND_ACK;
				control_wr_complete <= '1';
				updating_now <='0';
-- *************************** send ack back ***************************
			-- finished all the processing  send ack back
			WHEN G_SEND_ACK =>
				i_test <= x"0f";
				send_count <= x"0f";
				tx_csum <= x"00";
				get_data_state <= G_SEND_HEADER;
				data_out <= x"55";
			-- check TX fifo has space				
-- *************************** send header*******************************
			WHEN G_SEND_HEADER =>
				i_test <= x"10";
				get_data_state <= G_SEND_HEADER_0;
			-- send data
			WHEN G_SEND_HEADER_0 =>
				i_test <= x"11";
				ftdi_wr <='1';								
				get_data_state <= G_SEND_HEADER_1;
			-- wait for data to be sent
			WHEN G_SEND_HEADER_1 =>
				i_test <= x"12";
				IF ( tx_done = '1' ) THEN
					get_data_state <= G_SEND_HEADER_2;
				END IF;
			-- increment header byte count and csum
			WHEN G_SEND_HEADER_2 =>
				i_test <= x"13";
				ftdi_wr <='0';								
				IF ( send_count = x"0") THEN
					get_data_state <= G_SEND_BYTE_COUNT;
				ELSE
					send_count <= send_count-1;
					get_data_state <= G_SEND_HEADER_0;
				END IF;
-- *************************** send byte count **************************
			WHEN G_SEND_BYTE_COUNT =>
				i_test <= x"14";
				get_data_state <= G_SEND_BYTE_COUNT_0;
			-- check TX fifo has space				
			WHEN G_SEND_BYTE_COUNT_0 =>
				i_test <= x"15";
				get_data_state <= G_SEND_BYTE_COUNT_1;
			-- send byte count
			WHEN G_SEND_BYTE_COUNT_1 =>
				i_test <= x"16";
				data_out <= ACK_BYTE_COUNT;						
				get_data_state <= G_SEND_BYTE_COUNT_2;

				-- wait for data to be sent
			WHEN G_SEND_BYTE_COUNT_2 =>
--				tx_csum <= ACK_BYTE_COUNT;						
				i_test <= x"17";
				ftdi_wr <='1';								
				IF ( tx_done ='1' ) THEN
					get_data_state <= G_SEND_STATUS;
				END IF;
-- *************************** send status *******************************
			WHEN G_SEND_STATUS =>
				i_test <= x"18";
				ftdi_wr <='0';								
				i_control_address		<= x"00";
				send_count <=x"0f";
				packet_ram_addr <= ADDRESS_0;
				get_data_state <= G_SEND_STATUS_0;
				tx_csum <= x"00";
			-- wait for space in tx fifo
			WHEN G_SEND_STATUS_0 =>
				i_test <= x"19";
				get_data_state <= G_SEND_STATUS_1;
			-- get status data from instrumant
			WHEN G_SEND_STATUS_1 =>
				i_test <= x"1a";
				status_rd <='1';
				get_data_state <= G_SEND_STATUS_2;
			-- got data so put on internal bus				
			WHEN G_SEND_STATUS_2 =>
				i_test <= x"1b";
				-- if last byte send info on last packet transfer
				IF (send_count =x"1") THEN
					data_out <= x"a" & last_packet ;
				ELSE
					data_out <= status_data;
				END IF;
				tx_csum <= tx_csum + status_data;
				get_data_state <= G_SEND_STATUS_3;
			-- send data
			WHEN G_SEND_STATUS_3 =>
				i_test <= x"1c";
				ftdi_wr <='1';								
				get_data_state <= G_SEND_STATUS_4;
			-- wait for data to be sent
			WHEN G_SEND_STATUS_4 =>
				i_test <= x"1d";
				IF ( tx_done ='1' ) THEN
					get_data_state <= G_SEND_STATUS_5;
				END IF;
			-- increment byte count and check if done
			WHEN G_SEND_STATUS_5 =>
				i_test <= x"1e";
				ftdi_wr <='0';								
				status_rd <='0';			
				IF ( send_count = x"0") THEN
					get_data_state <= G_SEND_CSUM;
				ELSE
					i_control_address	<=  i_control_address + 1;
					send_count <= send_count-1;
					packet_ram_addr <= packet_ram_addr + 1;
					get_data_state <= G_SEND_STATUS_0;
				END IF;
-- *************************** send csum *******************************
			WHEN G_SEND_CSUM =>
				i_test <= x"1f";
				get_data_state <= G_SEND_CSUM_0;
				data_out <= tx_csum;
			-- check TX fifo has space				
			WHEN G_SEND_CSUM_0 =>
				i_test <= x"20";
				get_data_state <= G_SEND_CSUM_1;
			-- send header data
			WHEN G_SEND_CSUM_1 =>
				i_test <= x"21";
				ftdi_wr <='1';								
				get_data_state <= G_SEND_CSUM_2;
			-- wait for data to be sent
			WHEN G_SEND_CSUM_2 =>
				i_test <= x"22";
				IF ( tx_done ='1' ) THEN
					get_data_state <= G_IDLE;
				END IF;
			WHEN OTHERS =>
				get_data_state <= G_IDLE;
				i_test <= x"23";
		END CASE;
	END IF;
END PROCESS;	

packet_ram_inst : ram_dp
    generic map 
	(
		addr_bits		=> 6,
		data_bits		=> 8,
        register_out_flag	=> 1)
    port MAP
	(
		reset		=> reset,
		wr_clk		=> sysclk,
		wr_en	    => packet_ram_wr,
		wr_addr		=> packet_ram_addr,
		wr_data		=> ftdi_data,
	  	rd_clk		=> sysclk,
		rd_addr		=> packet_ram_addr,
		rd_data		=> i_packet_ram_data_out
	);

packet_ram_data_out <= i_packet_ram_data_out;
control_address <= i_control_address;

--  test stuff

--test(2) <= packet_found;
--test(3) <= find_packet_start;
test( 3 downto 0 ) <= i_test( 3 downto 0 );

--test(4) <= ftdi_data_rdy;
test(4) <= find_packet_start;
test(5) <= sync_count(0);
test(6) <= find_packet_start;

END rtl;












